drop specific procedure GET_ALERT_FROM_QUE
@

call sysproc.set_routine_opts('ISOLATION UR') @

CREATE PROCEDURE GET_ALERT_FROM_QUEUE ( IN i_user_id bigint,
                                        IN i_user_name CHAR(60),
                                        IN i_queue_id bigint,
                                        IN i_queue_sort_field VARCHAR(30),
                                        IN i_queue_sort_order VARCHAR(10),
                                       
                                      OUT o_alert_id BIGINT )
SPECIFIC GET_ALERT_FROM_QUE
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  GET_ALERT_FROM_QUE
--
--
-- Description:
--
--
-- Called By:
--     qry_alert_service
--
-- Calls:
--
--
-- Notes:
--
--
-- Usage:
--
--
-- History:    10/17/2005:          taken from FCMINT and stored in PRISM
--             2006/03/30   jicimi  changed -5 literal ( acct_gmt_offest )
--                                  to -8 (PST)
--             2006/04/18   jicimi  l_baseSql changed from varchar(2000) to
--                                  varchar(2500)
--             2006/04/17   jicimi  changed -8 literal ( acct_gmt_offest ) to
--                                  the value of fcm_alert.account_gmt_offset
--             2006/06/05   jicimi  added cms_acct_gmt_offset
--                                  update to fcm_alert sets pause_time = NULL
--                                    & pause_elapsed_time = 0
--                                  added logical_del_ind checks
--             2006/08/02           added alert_status ACTIVE check to
--                                    the predicate of the update statement
--                                  added the is null predicate to first subselect
--                                     in the odd event that the rank criterion
--                                     doesn't have a value
--
-- 			   2007/06/18           add curr_cust_hour UDF
--             18DEC2008    LEUSHE  S0553598 Removed checking on the SQLSTATE variable. Changed this by setting 
--                                  the v_sqlstate variable SQLSTATE, the conditionally checking v_sqlstate.
--                                  In db2 9.5, accessing the SQLSTATE variable, resets it. Thus, the IF would reset the SQLSTATE 
--                                  value. 
-- 			   2009/06/10           Updated for V3
--             2009/08/14    leushe Added the security check to ensure the user_name has access to update the alert
--             2010/04/26    leushe Added logic to return only alerts that are not suspended, and when gets an alert,  
--                                  adds the alert to the FCM_ALERT_SUSPEND table
-- 			   2011/03/24    leushe fixed fetch stmts that were throwing sqlstate 01503 because the fetch didn't have the l_grx_cust_num as a fetch into variable
--
------------------------------------------------------------------------

P1: BEGIN

    DECLARE l_alert_id bigint DEFAULT -1;
    DECLARE l_row_count INT DEFAULT 0;

    DECLARE v_sqlstate CHAR(5) DEFAULT '00000';
    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';
    DECLARE l_sort char(4) DEFAULT 'desc';

    DECLARE l_baseSql varchar(2500);
    DECLARE l_suspend char(5);
    DECLARE l_grx_cust_num char(34);
    DECLARE l_stmt STATEMENT;
    
    DECLARE c_stmt CURSOR for l_stmt;

--
--Check to see if suspended alerts is on
--
SELECT coalesce(SUBSTR(LCASE(PROPERTY_VALUE),1,5),'false')   INTO L_SUSPEND FROM FCM_PROPERTY WHERE PROPERTY_NAME = 
    	'analyst.alert.suspend.enabled';

    IF (L_SUSPEND is null) THEN
    	set L_SUSPEND = 'false';
    END IF;
    
    IF ( i_queue_sort_order = 'ASC' ) THEN
       SET l_sort = ' ';
    END IF;

	SET l_baseSql =
			'select alert_id,grx_cust_num
               from fcm_alert aa, 
               		fcm_queue q
               where aa.queue_id = ? and
                     check_mo_at_access( cast ( ? as BIGINT) ,aa.multi_org_id,aa.alert_type)=1 and
                      aa.queue_id = q.queue_id and
                     aa.alert_status = ''ACTIVE'' and
                     (aa.next_service_timestamp) <  (current timestamp - current timezone) and
                     (
                         (q.queue_call_superseded_ind = ''1'') or
                         ( curr_cust_hour( aa.gmt_offset) between
                                    q.queue_call_lower_bound and q.queue_call_upper_bound )
                      )
 					and not exists (select 1 from fcm_alert_suspend 
						where grx_cust_num = aa.grx_cust_num 
						      and suspend_user <> ? )
                order by aa.'||i_queue_sort_field||' '||l_sort||'
                fetch first 10 rows only
                for read only
                optimize for 1 row /*with UR*/';

		PREPARE l_stmt FROM l_baseSql;

		OPEN c_stmt using i_queue_id,i_user_id,i_user_name;

		fetch from c_stmt into l_alert_id,l_grx_cust_num;
		SET v_sqlstate = SQLSTATE;
		while(v_sqlstate = '00000') do

-- Security check for the user should have been done in the l_baseSql statement, so no need
-- to check security on this update
	    if (l_suspend = 'true') then
        insert into fcm_alert_suspend (grx_cust_num,alert_id,suspend_user,suspend_timestamp)
		select l_grx_cust_num,alert_id,i_user_name,current timestamp - current timezone 
			from fcm_alert
			where not exists (select 1 from fcm_alert_suspend 
						where grx_cust_num = l_grx_cust_num 
						      and suspend_user <> i_user_name 
							  and alert_id <> l_alert_id)
						and alert_id = l_alert_id 
						and alert_status='ACTIVE';
 
		
		 GET DIAGNOSTICS l_row_count = ROW_COUNT;
   	     if(l_row_count = 1) then
					
		  update fcm_alert set lstupdt_user = i_user_name,
              alert_status = 'CHECKOUT',
              alert_service_cnt = alert_service_cnt + 1,
              checkout_user = i_user_name,
              checkout_timestamp =  CURRENT TIMESTAMP - CURRENT TIMEZONE,
              user_service_timestamp = CURRENT TIMESTAMP - CURRENT TIMEZONE,
              lstupdt_timestamp = CURRENT TIMESTAMP - CURRENT TIMEZONE,
              lstupdt_nbr = lstupdt_nbr + 1,
              pause_timestamp = NULL,
              pause_elapsed_time = 0
          where
              alert_status = 'ACTIVE' and
              alert_id = l_alert_id
              
          /*with UR*/;

			GET DIAGNOSTICS l_row_count = ROW_COUNT;

			if(l_row_count = 1) then
				SET v_sqlstate = '00001';
			else
				set l_alert_id = -1;
				fetch from c_stmt into l_alert_id,l_grx_cust_num;
         		SET v_sqlstate = SQLSTATE;
			end if;
		  else
				set l_alert_id = -1;
				fetch from c_stmt into l_alert_id,l_grx_cust_num;
         		SET v_sqlstate = SQLSTATE;
			end if;
		else
				  update fcm_alert set lstupdt_user = i_user_name,
              alert_status = 'CHECKOUT',
              alert_service_cnt = alert_service_cnt + 1,
              checkout_user = i_user_name,
              checkout_timestamp =  CURRENT TIMESTAMP - CURRENT TIMEZONE,
              user_service_timestamp = CURRENT TIMESTAMP - CURRENT TIMEZONE,
              lstupdt_timestamp = CURRENT TIMESTAMP - CURRENT TIMEZONE,
              lstupdt_nbr = lstupdt_nbr + 1,
              pause_timestamp = NULL,
              pause_elapsed_time = 0
          where
              alert_status = 'ACTIVE' and
              alert_id = l_alert_id
              
          /*with UR*/;

			GET DIAGNOSTICS l_row_count = ROW_COUNT;

			if(l_row_count = 1) then
				SET v_sqlstate = '00001';
			else
				set l_alert_id = -1;
				fetch from c_stmt into l_alert_id,l_grx_cust_num;
         		SET v_sqlstate = SQLSTATE;
			end if;
		end if;
		end while;
		close c_stmt;

    /*
       regardless of success ( alert_id > 0) or failure ( alert_id = -1 )
       commit to release all resources  - necessary because of the select for update
    */
    COMMIT;



    set o_alert_id = l_alert_id;

END P1
@

--grant execute on specific procedure GET_ALERT_FROM_QUE to role approle
--@
